#include <Wire.h>

#define SHT40_I2C_ADDRESS 0x44

void setup() {
  Serial.begin(115200); // Initialize serial communication
  Wire.begin(8, 9);     // SDA on IO8, SCL on IO9

  Serial.println("Initializing SHT40 sensor...");

  // Check if the sensor is accessible on the I2C bus
  Wire.beginTransmission(SHT40_I2C_ADDRESS);
  if (Wire.endTransmission() != 0) {
    Serial.println("SHT40 not found. Check connections.");
    while (1) delay(1000); // Stop execution if the sensor is not found 
  }

  Serial.println("SHT40 found!");
}

void loop() {
  float temperature, humidity;

  // Read sensor data and print results
  if (readSHT40(&temperature, &humidity)) {
    Serial.printf("Temperature: %.2f °C\n", temperature);
    Serial.printf("Humidity: %.2f %%\n", humidity);
  } else {
    Serial.println("Failed to read from SHT40!");
  }

  delay(2000); // Wait 2 seconds before the next reading
}

bool readSHT40(float *temperature, float *humidity) {
  Wire.beginTransmission(SHT40_I2C_ADDRESS);
  Wire.write(0xFD); // Command for high-precision measurement
  if (Wire.endTransmission() != 0) return false;

  delay(10); // Wait for measurement to complete

  Wire.requestFrom(SHT40_I2C_ADDRESS, 6);
  if (Wire.available() == 6) {
    // Read raw temperature and humidity data
    uint16_t t_raw = (Wire.read() << 8) | Wire.read();
    Wire.read(); // Skip CRC
    uint16_t h_raw = (Wire.read() << 8) | Wire.read();
    Wire.read(); // Skip CRC

    // Convert raw data to physical values
    *temperature = -45.0f + 175.0f * (t_raw / 65535.0f);
    *humidity = 100.0f * (h_raw / 65535.0f);
    return true;
  }

  return false; // Return false if data is not available
}
